@@ -1,7 +1,7 @@
 //
 // IWrapper.cs: Redland Iterator wrapper class
 //
-// $Id: IWrapper.cs,v 1.3 2004/07/14 14:27:31 cmdjb Exp $
+// $Id: IWrapper.cs,v 1.4 2005/01/05 11:41:42 edd Exp $
 //
 // Author:
 //	Cesar Lopez Nataren (cesar@ciencias.unam.mx)
@@ -10,11 +10,12 @@
 //
 
 using System;
+using System.Runtime.InteropServices;
 
 namespace Redland {
 
 	public interface IWrapper {		
 
-		IntPtr Handle { get; }
+		HandleRef Handle { get; }
 	}
 }
