@@ -2,7 +2,7 @@
 #
 # Redland.pm - Redland top level Perl module
 #
-# $Id: Redland.pm,v 1.20 2004/07/28 10:03:50 cmdjb Exp $
+# $Id: Redland.pm,v 1.22 2005/02/22 21:28:51 cmdjb Exp $
 #
 # Copyright (C) 2000-2004 David Beckett - http://purl.org/net/dajobe/
 # Institute for Learning and Research Technology - http://www.ilrt.bris.ac.uk/
@@ -56,9 +56,13 @@ sub DESTROY ($) {
   &RDF::Redland::CORE::librdf_perl_world_finish();
 }
 
-sub message ($$) {
-  my($type,$message)=@_;
-  if($type == 0) {
+sub message ($$$$$$$$$) {
+  if(ref $RDF::Redland::Log_Sub) {
+    return $RDF::Redland::Log_Sub->(@_);
+  }
+
+  my($code, $level, $facility, $message, $line, $column, $byte, $file, $uri)=@_;
+  if($level > 3) {
     if(ref $RDF::Redland::Error_Sub) {
       return $RDF::Redland::Error_Sub->($message);
     } else {
@@ -86,6 +90,8 @@ $Error_Sub=undef;
 
 $Warning_Sub=undef;
 
+$Log_Sub=undef;
+
 sub set_error_handler ($) {
   $Error_Sub=shift;
 }
@@ -94,6 +100,15 @@ sub set_warning_handler ($) {
   $Warning_Sub=shift;
 }
 
+# Internal
+sub set_log_handler ($) {
+  $Log_Sub=shift;
+}
+
+sub reset_log_handler () {
+  $Log_Sub=undef;
+}
+
 
 =pod
 
