@@ -1,23 +1,26 @@
 # 
 # RDF.py - Redland Python RDF module
 #
-# $Id: RDF.py,v 1.84 2004/07/23 10:02:59 cmdjb Exp $
+# $Id: RDF.py,v 1.100 2005/02/24 14:11:19 cmdjb Exp $
 #
-# Copyright (C) 2000-2004 David Beckett - http://purl.org/net/dajobe/
+# Copyright (C) 2000-2005 David Beckett - http://purl.org/net/dajobe/
 # Institute for Learning and Research Technology - http://www.ilrt.bris.ac.uk/
 # University of Bristol - http://www.bristol.ac.uk/
 # 
-# This package is Free Software or Open Source available under the
-# following licenses (these are alternatives):
-#   1. GNU Lesser General Public License (LGPL)
-#   2. GNU General Public License (GPL)
-#   3. Mozilla Public License (MPL)
+# This package is Free Software and part of Redland http://librdf.org/
+# 
+# It is licensed under the following three licenses as alternatives:
+#   1. GNU Lesser General Public License (LGPL) V2.1 or any newer version
+#   2. GNU General Public License (GPL) V2 or any newer version
+#   3. Apache License, V2.0 or any newer version
+#
+# This one file RDF.py is also available under the following license:
+#
+#   4. BSD License without advertising (aka MIT license)
+#      from http://www.opensource.org/licenses/mit-license.html
 #
-# and as an exception this one file RDF.py is also available under the
-#   4. BSD License without advertising (MIT license - from
-#        http://www.opensource.org/licenses/mit-license.html)
 #      -------------------------------------------------------------------
-#      Copyright (c) 2003-2004, David Beckett, ILRT, University of Bristol
+#      Copyright (c) 2000-2005, David Beckett, ILRT, University of Bristol
 #
 #      Permission is hereby granted, free of charge, to any person
 #      obtaining a copy of this software and associated documentation
@@ -40,10 +43,12 @@
 #      SOFTWARE.
 #     --------------------------------------------------------------------
 #
+# You may not use this file except in compliance with at least one of
+# the above four licenses.
 # 
 # See LICENSE.html or LICENSE.txt at the top of this package for the
-# full license terms.
-# 
+# complete terms and further detail along with the license texts for
+# the licenses in COPYING.LIB, COPYING and LICENSE-2.0.txt respectively.
 # 
 #
 
@@ -84,6 +89,8 @@ __all__ = [ "Node",
             "Model",
             "Iterator",
             "Serializer",
+            "NTriplesSerializer",
+            "RDFXMLSerializer",
             "Stream",
             "Storage",
             "MemoryStorage",
@@ -94,12 +101,14 @@ __all__ = [ "Node",
             "NTriplesParser",
             "NS",
             "Query",
+            "RDQLQuery",
+            "SPARQLQuery",
             "debug"]
 
-__version__ = "1.0"
+__version__ = "1.1"
 
 # For pydoc
-__date__ = '$Date: 2004/07/23 10:02:59 $'
+__date__ = '$Date: 2005/02/24 14:11:19 $'
 __author__ = 'Dave Beckett - http://purl.org/net/dajobe, Edd Dumbill <edd@usefulinc.com> and Matt Biddulph <mb@picdiary.com>'
 
 # Package variables [globals]
@@ -150,27 +159,10 @@ def node_type_name(num):
   raise NodeTypeError('Unknown node type number %d' % num)
 
 def message_handler (type, message):
-  """Internal message dispatcher from Redland to python"""
-  global _debug
-  if _debug:
-      print "message_handler: type, message = ",type,message
-  if type == 0:
-    raise RedlandError(message)
-  else:
-    import warnings
-    warnings.warn(message, UserWarning, stacklevel=3)
+  raise RedlandError("Obsolete method")
 
 def set_message_handler(handler):
-  """Set the Redland message handler for Python.  It takes
-     a single function that takes (integer, string) arguments.
-
-     If it returns an non-zero integer, Redland will consider
-     the message handled and not display it using the default
-     handler."""
-  import Redland_python
-
-  Redland_python.set_callback(handler)
-
+  raise RedlandError("Obsolete method")
 
 class World(object):
   """Redland Initialisation class.
@@ -185,8 +177,6 @@ class World(object):
     Redland.librdf_world_open(self._world)
 
     Redland.librdf_python_world_init(self._world)
-    import Redland_python
-    Redland_python.set_callback(message_handler)
 
   def __del__(self):
     """Destroy RDF World object (destructor)."""
@@ -259,7 +249,7 @@ class Node(object):
       literal     - create a literal node from a literal string   
         datatype     - the datatype URI
         is_wf_xml    - the literal is XML (alternative to datatype)
-        xml_language - the literal XML language
+        language     - the literal XML language
       blank       - create a resource node from with a blank node identiifer
       node        - copy a node
     """
@@ -273,6 +263,9 @@ class Node(object):
     if arg is not None:
       if type(arg) is str:
         args['literal'] = arg
+      elif type(arg) is unicode:
+        import Redland_python
+        args['literal'] = Redland_python.unicode_to_bytes(arg)
       elif type(arg) is Uri:
         args['uri'] = arg
       elif type(arg) is Node:
@@ -291,6 +284,8 @@ class Node(object):
     elif args.has_key('literal'):
       if args.has_key('xml_language'):
         xml_language=args['xml_language']
+      elif args.has_key('language'):
+        xml_language=args['language']
       else:
         xml_language=""
       if args.has_key('is_wf_xml'):
@@ -354,14 +349,14 @@ class Node(object):
     if dt_uri:
       dt_uri=Uri(string=Redland.librdf_uri_to_string(dt_uri))
     val={
-        'string': Redland.librdf_node_get_literal_value(self._node),
+        'string': unicode(Redland.librdf_node_get_literal_value(self._node), 'utf-8'),
         'language': Redland.librdf_node_get_literal_value_language(self._node),
         'datatype': dt_uri
         }
     return val
 
   literal_value = property(_get_literal_value,
-          doc = "A dictionary containing the value of the node literal")
+          doc = "A dictionary containing the value of the node literal with keys string, language and datatype")
 
   def _get_blank_identifier(self):
     if not self.is_blank():
@@ -380,6 +375,13 @@ class Node(object):
     else:
       return Redland.librdf_node_to_string(self._node)
 
+  def __unicode__(self):
+    """Get a Unicode string representation of an RDF Node."""
+    if self._node is None:
+      raise RedlandError("Node is invalid") 
+    else:
+      return unicode(Redland.librdf_node_to_string(self._node), 'utf-8')
+
   def __eq__(self,other):
     """Equality of an RDF Node compared to another RDF Node."""
     if other is None:
@@ -462,6 +464,9 @@ class Statement(object):
       else:
         if type(subject) is Uri or type(subject) is str:
           subject = Node(subject)
+        elif type(subject) is unicode:
+          import Redland_python
+          subject = Node(Redland_python.unicode_to_bytes(subject))
         s = Redland.librdf_new_node_from_node(subject._node)
 
       if predicate is None:
@@ -469,6 +474,9 @@ class Statement(object):
       else:
         if type(predicate) is Uri or type(predicate) is str:
           predicate = Node(predicate)
+        elif type(predicate) is unicode:
+          import Redland_python
+          predicate = Node(Redland_python.unicode_to_bytes(predicate))
         p = Redland.librdf_new_node_from_node(predicate._node)
 
       if object is None:
@@ -476,6 +484,9 @@ class Statement(object):
       else:
         if type(object) is Uri or type(object) is str:
           object = Node(object)
+        elif type(object) is unicode:
+          import Redland_python
+          object = Node(Redland_python.unicode_to_bytes(object))
         o = Redland.librdf_new_node_from_node(object._node)
 
       self._statement=Redland.librdf_new_statement_from_nodes(
@@ -544,6 +555,12 @@ class Statement(object):
     else:
       return Redland.librdf_statement_to_string(self._statement)
 
+  def __unicode__(self):
+    if self._statement is None:
+      raise RedlandError("Statement is invalid")
+    else:
+      return unicode(Redland.librdf_statement_to_string(self._statement), 'utf-8')
+
   def __eq__(self,other):
     """Equality of an RDF Statement compared to another RDF Statement."""
     if other is None:
@@ -693,11 +710,13 @@ Create a model using an in-memory storage.
     # C methods is a recipe for unexplained core dumps if any of the
     # nodes are null or invalid.
     if datatype is not None:
-      datatype = datatype._reduri
+      rdatatype = datatype._reduri
+    else:
+      rdatatype = None
     subject_copy = Redland.librdf_new_node_from_node(subject._node)
     predicate_copy = Redland.librdf_new_node_from_node(predicate._node)
     return Redland.librdf_model_add_typed_literal_statement(self._model,
-        subject_copy, predicate_copy, string, xml_language, datatype)
+        subject_copy, predicate_copy, string, xml_language, rdatatype)
 
   def add_statement(self,statement,context=None):
     """Add the Statement to the Model with optional context Node.
@@ -864,6 +883,9 @@ Create a model using an in-memory storage.
       predicate = Node(predicate)
     if type(target) is Uri or type(target) is str:
       target = Node(target)
+    elif type(target) is unicode:
+      import Redland_python
+      target = Node(Redland_python.unicode_to_bytes(target))
 
     my_iterator = Redland.librdf_model_get_sources(self._model,
                                                    predicate._node,
@@ -894,6 +916,9 @@ Create a model using an in-memory storage.
       source = Node(source)
     if type(target) is Uri or type(target) is str:
       target = Node(target)
+    elif type(target) is unicode:
+      import Redland_python
+      target = Node(Redland_python.unicode_to_bytes(target))
 
     my_iterator=Redland.librdf_model_get_arcs(self._model, source._node,
                                               target._node)
@@ -948,6 +973,9 @@ Create a model using an in-memory storage.
       predicate = Node(predicate)
     if type(target) is Uri or type(target) is str:
       target = Node(target)
+    elif type(target) is unicode:
+      import Redland_python
+      target = Node(Redland_python.unicode_to_bytes(target))
 
     my_node=Redland.librdf_model_get_source(self._model, predicate._node,
                                             target._node)
@@ -963,6 +991,9 @@ Create a model using an in-memory storage.
       source = Node(source)
     if type(target) is Uri or type(target) is str:
       target = Node(target)
+    elif type(target) is unicode:
+      import Redland_python
+      target = Node(Redland_python.unicode_to_bytes(target))
 
     my_node=Redland.librdf_model_get_arc(self._model, source._node,
                                          target._node)
@@ -1019,21 +1050,79 @@ Create a model using an in-memory storage.
       value=Node(literal=value)
     Redland.librdf_model_set_feature(self._model,uri._reduri,value._node)
 
-  def load(self, uri, name="", mime_type="", type_uri=None):
+  def load(self, uri, name="", mime_type="", type_uri=None, handler=None):
     """Load triples into the Model from a URI in a syntax.
 
        If no parser name is given, the parser to use is guessed.
+
+       If handler is given, an error handler with the signature
+         def handler(code, level, facility, message, line, column, byte, file, uri)
+       is called.
    """
     if type(uri) is str:
       uri = Uri(string=uri)
-    if uri is not None:
-      uri = uri._reduri
+    elif type(uri) is unicode:
+      import Redland_python
+      uri = Uri(string=Redland_python.unicode_to_bytes(uri))
+    if uri is None:
+      raise TypeError("uri must be a string or RDF.Uri")
+    ruri = uri._reduri
+
     if type(type_uri) is str:
       type_uri = Uri(string=type_uri)
+    elif type(type_uri) is unicode:
+      import Redland_python
+      type_uri = Uri(string=Redland_python.unicode_to_bytes(type_uri))
     if type_uri is not None:
-      type_uri = type_uri._reduri
-    Redland.librdf_model_load(self._model, uri, name, mime_type, type_uri)
-    
+      rtype_uri = type_uri._reduri
+    else:
+      rtype_uri = None
+
+    if handler is not None:
+      import Redland_python
+      Redland_python.set_callback(handler)
+
+    rc=Redland.librdf_model_load(self._model, ruri, name, mime_type, rtype_uri)
+
+    if handler is not None:
+      import Redland_python
+      Redland_python.reset_callback()
+
+    return rc
+
+  def to_string(self, base_uri=None, name="", mime_type="", type_uri=None):
+    """Serialize the Model to a syntax.
+
+       print model.to_string(base_uri="http://example.org/base");
+
+       If no serializer name is given, the default serializer RDF/XML is used.
+   """
+    if type(base_uri) is str:
+      base_uri = Uri(string=base_uri)
+    elif type(base_uri) is unicode:
+      import Redland_python
+      base_uri = Uri(string=Redland_python.unicode_to_bytes(base_uri))
+    if base_uri is not None:
+      rbase_uri = base_uri._reduri
+    else:
+      rbase_uri = None
+    if type(type_uri) is str:
+      type_uri = Uri(string=type_uri)
+    elif type(type_uri) is unicode:
+      import Redland_python
+      type_uri = Uri(string=Redland_python.unicode_to_bytes(type_uri))
+    if type_uri is not None:
+      rtype_uri = type_uri._reduri
+    else:
+      rtype_uri = None
+    return Redland.librdf_model_to_string(self._model, rbase_uri, name, mime_type, rtype_uri)
+
+  def __str__(self):
+    return self.to_string()
+
+  def __unicode__(self):
+    return unicode(self.to_string(), 'utf-8')
+
   def execute(self,query):
     results = Redland.librdf_model_query_execute(self._model,query._query)
     if results is not None:
@@ -1315,24 +1404,72 @@ class Storage(object):
     Create a new RDF Storage using any of these forms
 
   s1=RDF.Storage(storage_name="name")
-Create a Storage with the given name.  Currently the built in
-storages are "memory" and "hashes".  "hashes" takes extra
-arguments passed in the field options_string, some of which are
-required:
-  options_string="hash-type='memory',new='yes',write='yes'"
-    hash-type - required and can be the name of any Hash type supported.
-      'memory' is always present, and 'bdb' is available
-      when BerkeleyDB is compiled in.
-    new - optional and takes a boolean value (default false)
-      If true, it allows updating of an existing Storage 
-    write - optional and takes a boolean value (default true)
-      If false, the Storage is opened read-only and for file-based
-      Storages or those with locks, may be shared-read.
 
+Create a Storage of the given type.  Currently the built in storage
+names that are always present are 'memory', 'hashes', 'file' and
+'uri'.  'bdb' is available when Sleepycat / BerkeleyDB is compiled
+in, 'mysql' when MySQL is compiled in, and 'sqlite' when SQLite is
+compiled in.  If storage_name is omitted, it defaults to 'memory'.
+
+The argument 'name' can be used when the storage needs a name
+to operate, such as used for a filename or URI:
+
+  s1=RDF.Storage(storage_name="file", name='/filename')
+
+  s2=RDF.Storage(storage_name="uri", name='http://rdf.example.org/')
+
+
+The argument options_string allows additional store-specific options
+to be given, some of which are required by certain stores.  This uses
+the following form:
+
+  s3=RDF.Storage(storage_name="name", name='abc',
+                 options_string="key1='value1', key2='value2', ...")
+
+for multiple key/value option pairs, option values are always
+surrounded by single quotes.
+
+The common options are:
+  new - optional and takes a boolean value (default false)
+    If true, it deletes any existing store and creates a new one
+    otherwise if false (default) open an existing store.
+
+  write - optional and takes a boolean value (default true)
+    If true (default) the Storage is opened read-write otherwise
+    if false the storage is opened read-only and for file-based
+    Storages or those with locks, may be opened with shared-readers.
+
+Some storage types have additional options:
+
+storage_name 'hashes' has options:
+  hash-type -  the name of any hash type supported.
+    'memory' (default), 'file' hash types are always present. 'bdb' is
+    available when BerkeleyDB is compiled in,
+  dir - the directory name to create the files in (default '.')
+  mode - the file creation mode (default 0644 octal)
+
+storage_name 'mysql' has options:
+  host - required MySQL database hostname
+  port - optional MySQL database port (defaults to 3306)
+  database - required MySQL database name
+  user - required MySQL database user
+  password - required MySQL database password
+      
 The other form is:
-  s2=RDF.Storage(storage=s1)
+  s4=RDF.Storage(storage=s1)
 Copy an existing Storage s1.
 
+
+Note: there are convience classes to create a memory storage
+  s5=RDF.MemoryStorage()
+
+and Hash storage:
+  # memory hash
+  s6=RDF.HashStorage('abc')
+
+  # specified bdb hash stored in files named 'def'*
+  s7=RDF.HashStorage('def', options="hash-type='bdb'")
+
     """
     global _world
     global _debug    
@@ -1444,6 +1581,9 @@ Copy an existing URI uri1.
     if arg is not None:
       if type(arg) is str:
         args['string'] = arg
+      elif type(arg) is unicode:
+        import Redland_python
+        args['string'] = Redland_python.unicode_to_bytes(arg)
       elif type(arg) is Uri:
         args['uri'] = arg
 
@@ -1473,6 +1613,10 @@ Copy an existing URI uri1.
     """Get a string representation of an RDF URI."""
     return Redland.librdf_uri_to_string(self._reduri)
 
+  def __unicode__(self):
+    """Get a Unicode string representation of an RDF URI."""
+    return unicode(Redland.librdf_uri_to_string(self._reduri), 'utf-8')
+
   def __hash__(self):
     return hash(self._reduri)
   
@@ -1505,7 +1649,7 @@ class Parser(object):
   for the RDF/XML syntax.
   """
   
-  def __init__(self, name="rdfxml", mime_type="application/rdf+xml", uri=None):
+  def __init__(self, name="rdfxml", mime_type="", uri=None):
     """Create an RDF Parser (constructor).
 
 Create a new RDF Parser for a particular syntax.  The parser is
@@ -1525,9 +1669,13 @@ optional.  When any are given, they must all match.
       print "Creating RDF.Parser name=",name,"mime_type=",mime_type, "uri=",uri
 
     if uri is not None:
-      uri=uri._reduri
+      ruri=uri._reduri
+    else:
+      ruri=None
 
-    self._parser=Redland.librdf_new_parser(_world._world, name, mime_type, uri)
+    self._parser=Redland.librdf_new_parser(_world._world, name, mime_type, ruri)
+    if self._parser is None:
+      raise RedlandError("Failed to create parser "+name)
 
   def __del__(self):
     global _debug    
@@ -1537,15 +1685,19 @@ optional.  When any are given, they must all match.
       Redland.librdf_free_parser(self._parser)
 
   def parse_as_stream(self, uri, base_uri=None):
-    """"Return a Stream of Statements from parsing the content at
-        (file: only at present) URI, for the optional base URI
-        or None if the parsing fails.
+    """Return a Stream of Statements from parsing the content at
+        a URI for the optional base URI or None if the parsing fails.
+
+        (This depends on what URI support raptor provides to redland)
         
           for statement in parser.parse_as_stream("http://localhost/r.rdf"):
               print statement
     """
     if type(uri) is str:
       uri = Uri(string=uri)
+    elif type(uri) is unicode:
+      import Redland_python
+      uri = Uri(string=Redland_python.unicode_to_bytes(uri))
     if base_uri is None:
       base_uri=uri
     my_stream=Redland.librdf_parser_parse_as_stream(self._parser,
@@ -1555,7 +1707,7 @@ optional.  When any are given, they must all match.
     return Stream(my_stream, self)
 
   def parse_string_as_stream(self, string, base_uri):
-    """"Return a Stream of Statements from parsing the content in
+    """Return a Stream of Statements from parsing the content in
         string with the required base URI or None if the parsing fails.
 
           for statement in parser.parse_string_as_stream(rdfstring, base_uri):
@@ -1563,6 +1715,9 @@ optional.  When any are given, they must all match.
     """
     if type(base_uri) is str:
       base_uri = Uri(string=base_uri)
+    elif type(base_uri) is unicode:
+      import Redland_python
+      base_uri = Uri(string=Redland_python.unicode_to_bytes(base_uri))
     if base_uri is None:
       raise RedlandError("A base URI is required when parsing a string")
     my_stream=Redland.librdf_parser_parse_string_as_stream(self._parser,
@@ -1571,33 +1726,77 @@ optional.  When any are given, they must all match.
       return None
     return Stream(my_stream, self)
 
-  def parse_into_model(self, model, uri, base_uri=None):
-    """"Parse into the Model model from the content at
-        (file: only at present) URI, for the optional base URI.
+  def parse_into_model(self, model, uri, base_uri=None, handler=None):
+    """Parse into the Model model from the content at the URI, for
+       the optional base URI.
         
-          parser.parse_into_model(model, "file:./foo.rdf",
-                                  "http://example.com/foo.rdf")
+       If handler is given, an error handler with the signature
+         def handler(code, level, facility, message, line, column, byte, file, uri)
+       is called.
+
+       parser.parse_into_model(model, "file:./foo.rdf",
+                               "http://example.com/foo.rdf")
     """
 
     if type(uri) is str:
       uri = Uri(string = uri)
+    elif type(uri) is unicode:
+      import Redland_python
+      uri = Uri(string=Redland_python.unicode_to_bytes(uri))
+    if type(base_uri) is str:
+      base_uri = Uri(string = base_uri)
+    elif type(base_uri) is unicode:
+      import Redland_python
+      base_uri = Uri(string=Redland_python.unicode_to_bytes(base_uri))
     if base_uri is None:
       base_uri = uri
+
+    if handler is not None:
+      import Redland_python
+      Redland_python.set_callback(handler)
+
     try:
       r = Redland.librdf_parser_parse_into_model(self._parser,
         uri._reduri, base_uri._reduri, model._model)
     except RedlandError, err:
       print "Caught error",err
       raise err
+
+    if handler is not None:
+      import Redland_python
+      Redland_python.reset_callback()
+
     return 1
 
-  def parse_string_into_model(self, model, string, base_uri):
-    """"Parse into the Model model from the content ain string
-        with the required base URI"""
+  def parse_string_into_model(self, model, string, base_uri, handler = None):
+    """Parse into the Model model from the content string
+        with the required base URI
+
+       If handler is given, an error handler with the signature
+         def handler(code, level, facility, message, line, column, byte, file, uri)
+       is called.
+
+    """
+    if type(base_uri) is str:
+      base_uri = Uri(string = base_uri)
+    elif type(base_uri) is unicode:
+      import Redland_python
+      base_uri = Uri(string=Redland_python.unicode_to_bytes(base_uri))
     if base_uri is None:
       raise RedlandError("A base URI is required when parsing a string")
-    return Redland.librdf_parser_parse_string_into_model(self._parser,
-      string, base_uri._reduri, model._model)
+
+    if handler is not None:
+      import Redland_python
+      Redland_python.set_callback(handler)
+
+    rc=Redland.librdf_parser_parse_string_into_model(self._parser, string,
+                                           base_uri._reduri, model._model)
+
+    if handler is not None:
+      import Redland_python
+      Redland_python.reset_callback()
+
+    return rc
 
   def get_feature(self, uri):
     """Return the Node value of Parser feature URI uri"""
@@ -1619,6 +1818,10 @@ optional.  When any are given, they must all match.
 
 # end class Parser
 
+class RDFXMLParser(Parser):
+  def __init__(self, uri = None):
+    return Parser.__init__(self, name = "rdfxml", mime_type="", uri=uri)
+
 class NTriplesParser(Parser):
   def __init__(self, uri = None):
     return Parser.__init__(self, name = "ntriples", mime_type="text/plain", uri=uri)
@@ -1627,13 +1830,18 @@ class TurtleParser(Parser):
   def __init__(self, uri = None):
     return Parser.__init__(self, name = "turtle", mime_type="application/x-turtle", uri=uri)
 
+class RSSTagSoupParser(Parser):
+  def __init__(self, uri = None):
+    return Parser.__init__(self, name = "rss-tag-soup", mime_type="", uri=uri)
+
+
 class Query(object):
   """Redland Query interface class
 
   import RDF
 
   q1 = RDF.Query("SELECT ?a ?c WHERE (?a dc:title ?c) USING dc FOR <http://purl.org/dc/elements/1.1/>")
-  q2 = RDF.Query("- - -",name="triples")
+  q2 = RDF.Query("- - -", name="triples")
 
   results=q1.execute(model)
   for result in results:
@@ -1644,15 +1852,21 @@ class Query(object):
     print statement
 
   """
-  def __init__(self,querystring,base_uri=None,query_language="rdql"):
+  def __init__(self, querystring, base_uri=None, query_language="rdql", query_uri=None):
+    if query_uri is not None:
+      ruri = query_uri._reduri
+    else:
+      ruri = None
     if base_uri is not None:
-      base_uri = base_uri._reduri
+      rbase_uri = base_uri._reduri
+    else:
+      rbase_uri = None
     global _world
     global _debug    
     if _debug:
-      print "Creating query for language '"+query_language+"', base '"+str(base_uri)+"': "+querystring
+      print "Creating query for language '"+query_language+"', base '"+str(rbase_uri)+"': "+querystring
 
-    self._query = Redland.librdf_new_query(_world._world, query_language, base_uri, querystring)
+    self._query = Redland.librdf_new_query(_world._world, query_language, ruri, querystring, rbase_uri)
     self.result_stream = None
     self.been_run = False
 
@@ -1674,6 +1888,19 @@ class Query(object):
 # end class Query
 
 
+class RDQLQuery(Query):
+  """Redland RDQL Query class"""
+  def __init__(self, querystring, base_uri=None):
+    return Query.__init__(self, querystring = querystring, base_uri = base_uri,
+            query_language = "rdql")
+
+class SPARQLQuery(Query):
+  """Redland SPARQL Query class"""
+  def __init__(self, querystring, base_uri=None):
+    return Query.__init__(self, querystring = querystring, base_uri = base_uri,
+            query_language = "sparql")
+
+
 class QueryResults(object):
   """Redland Query results class
 
@@ -1687,14 +1914,30 @@ class QueryResults(object):
     self._results = results
     self.first = True
 
+  def is_bindings(self):
+    """Test if the query results format is variable bindings"""
+    return (Redland.librdf_query_results_is_bindings(self._results) != 0)
+
+  def is_boolean(self):
+    """Test if the query results format is a boolean"""
+    return (Redland.librdf_query_results_is_boolean(self._results) != 0)
+
+  def is_graph(self):
+    """Test if the query results format is an RDF graph"""
+    return (Redland.librdf_query_results_is_graph(self._results) != 0)
+
   def __iter__(self):
     return self
 
   def __len__(self):
+    """Get the number of query results returned so far"""
     return Redland.librdf_query_results_get_count(self._results)
 
   # Iterator method
   def next(self):
+    """Get the next variable binding result"""
+    if not self.is_bindings():
+      raise RedlandError("Query result is not in variable bindings format")
     if self.first:
       self.first = False
     else:
@@ -1713,37 +1956,115 @@ class QueryResults(object):
 
     return results
 
+  def finished(self):
+    """Test if reached the last variable binding result"""
+    if not self.is_bindings():
+      raise RedlandError("Query result is not in variable bindings format")
+    return (Redland.librdf_query_results_finished(self._results) != 0)
+
   def as_stream(self):
+    """Return the query results as a stream of triples (RDF.Statement)"""
+    if not self.is_graph():
+      raise RedlandError("Query result is not in RDF graph format")
     s=Redland.librdf_query_results_as_stream(self._results)
     if s is not None:
       return Stream(s, self)
     else:
       return None
 
-  def finished(self):
-    return Redland.librdf_query_results_finished(self._results)
+  def get_boolean(self):
+    """Get the boolean query result"""
+    if not self.is_boolean():
+      raise RedlandError("Query result is not in boolean format")
+    return (Redland.librdf_query_results_get_boolean(self._results) != 0)
 
   def get_binding_value(self, offset):
+    """Get the value of a variable binding by offset"""
+    if not self.is_bindings():
+      raise RedlandError("Query result is not in variable bindings format")
     n=Redland.librdf_query_results_get_binding_value(self._results, offset)
     return Node(from_object=n, do_not_copy=1)
 
   def get_binding_name(self, offset):
+    """Get the name of a variable binding by offset"""
+    if not self.is_bindings():
+      raise RedlandError("Query result is not in variable bindings format")
     return Redland.librdf_query_results_get_binding_name(self._results, offset)
 
   def get_binding_value_by_name(self, name):
+    """Get the value of a variable binding by variable name"""
+    if not self.is_bindings():
+      raise RedlandError("Query result is not in variable bindings format")
     n=Redland.librdf_query_results_get_binding_value_by_name(self._results, name)
     return Node(from_object=n, do_not_copy=1)
 
   def get_bindings_count(self):
+    """Get the number of variable bindings in the query result"""
+    if not self.is_bindings():
+      raise RedlandError("Query result is not in variable bindings format")
     return Redland.librdf_query_results_get_bindings_count(self._results)
 
   def __del__(self):
     global _debug    
     if _debug:
-      print "Destroying RDF.QueryResults"
+      print "Destroying RDF.QueryResults"""
     if self._results:
       Redland.librdf_free_query_results(self._results)
 
+  def to_file(self, name, base_uri=None):
+    """Serialize to filename name using the optional base URI."""
+    if type(base_uri) is str:
+      base_uri = Uri(string = base_uri)
+    elif type(base_uri) is unicode:
+      import Redland_python
+      base_uri = Uri(string=Redland_python.unicode_to_bytes(base_uri))
+    if base_uri is not None:
+      rbase_uri = base_uri._reduri
+    else:
+      rbase_uri = None
+    return Redland.librdf_query_results_to_file(self._results, name, rbase_uri)
+
+  def to_string(self, format_uri=None, base_uri=None):
+    """Serialize to a string syntax format_uri using the optional base URI."""
+    if self.is_graph():
+      tmpmodel = Model(MemoryStorage())
+      tmpmodel.add_statements(self.as_stream())
+      serializer = RDF.Serializer()
+      return serializer.serialize_model_to_string(tmpmodel, base_uri)
+
+    if self.is_boolean():
+      return str(self.get_boolean())
+
+    if not self.is_bindings():
+      raise RedlandError("Unknown query result format cannot be written as a string")
+    
+    if type(format_uri) is str:
+      format_uri = Uri(string = format_uri)
+    elif type(format_uri) is unicode:
+      import Redland_python
+      format_uri = Uri(string=Redland_python.unicode_to_bytes(format_uri))
+    else:
+      format_uri = Uri(string="http://www.w3.org/TR/2004/WD-rdf-sparql-XMLres-20041221/")
+    if format_uri is not None:
+      rformat_uri = format_uri._reduri
+    else:
+      rformat_uri = None
+    if type(base_uri) is str:
+      base_uri = Uri(string = base_uri)
+    elif type(base_uri) is unicode:
+      import Redland_python
+      base_uri = Uri(string=Redland_python.unicode_to_bytes(base_uri))
+    if base_uri is not None:
+      rbase_uri = base_uri._reduri
+    else:
+      rbase_uri = None
+      return Redland.librdf_query_results_to_string(self._results,
+                                                    rformat_uri, rbase_uri)
+
+  def __str__(self):
+    """Serialize to string syntax format."""
+    return self.to_string()
+
 
 # end class QueryResults
 
@@ -1752,14 +2073,27 @@ class Serializer(object):
   """ Redland Syntax Serializer Class
 
   import RDF
-  ser1=RDF.Serializer(mime_type="application/rdf+xml")
+  ser1=RDF.Serializer()
+
+  ser2=RDF.Serializer(mime_type="application/rdf+xml")
+
+  ser3=RDF.Serializer(name="ntriples")
 
   A class for turning a Model into a syntax serialization (at present
   only to local files).
   """
   
-  def __init__(self, name="", mime_type="application/rdf+xml", uri=None):
-    """Create an RDF Serializer (constructor)."""
+  def __init__(self, name="", mime_type="", uri=None):
+    """Create an RDF Serializer (constructor).
+
+    The arguments name, mime_type and uri are all optional and
+    when omitted the default serialization syntax is used.  If
+    any arguments are given, they must all match for an appropriate
+    syntax to be chosen.  For example, RDF/XML has a MIME type of
+    'application/rdf+xml' so this can be given with the mime_type
+    argument, however the N-Triples has none, so the 'ntriples' name
+    must be used.  Most syntaxes have URIs.
+    """
     global _world
     global _debug    
     if _debug:
@@ -1767,10 +2101,12 @@ class Serializer(object):
         "uri=",uri
 
     if uri is not None:
-      uri = uri._reduri
+      ruri = uri._reduri
+    else:
+      ruri = None
 
     self._serializer=Redland.librdf_new_serializer(_world._world, name,
-      mime_type, uri)
+      mime_type, ruri)
 
   def __del__(self):
     global _debug    
@@ -1782,10 +2118,30 @@ class Serializer(object):
   def serialize_model_to_file(self, name, model, base_uri=None):
     """Serialize to filename name the Model model using the
        optional base URI."""
+    if type(base_uri) is str:
+      base_uri = Uri(string = base_uri)
+    elif type(base_uri) is unicode:
+      import Redland_python
+      base_uri = Uri(string=Redland_python.unicode_to_bytes(base_uri))
     if base_uri is not None:
-      base_uri = base_uri._reduri
+      rbase_uri = base_uri._reduri
+    else:
+      rbase_uri = None
     return Redland.librdf_serializer_serialize_model_to_file(self._serializer,
-      name, base_uri, model._model)
+      name, rbase_uri, model._model)
+
+  def serialize_model_to_string(self, model, base_uri=None):
+    """Serialize to a string using the optional base URI."""
+    if type(base_uri) is str:
+      base_uri = Uri(string = base_uri)
+    elif type(base_uri) is unicode:
+      import Redland_python
+      base_uri = Uri(string=Redland_python.unicode_to_bytes(base_uri))
+    if base_uri is not None:
+      rbase_uri = base_uri._reduri
+    else:
+      rbase_uri = None
+    return Redland.librdf_serializer_serialize_model_to_string(self._serializer, rbase_uri, model._model)
 
   # TODO: features could usefully be implemented as a collection
 
@@ -1794,6 +2150,9 @@ class Serializer(object):
     if not isinstance(uri, Uri):
       if type(uri) is str:
         uri = Uri(string=uri)
+      elif type(uri) is unicode:
+        import Redland_python
+        uri = Uri(string=Redland_python.unicode_to_bytes(uri))
       else:
         raise TypeError("uri must be string or RDF.Uri")
 
@@ -1804,6 +2163,9 @@ class Serializer(object):
     if not isinstance(uri, Uri):
       if type(uri) is str:
         uri = Uri(string=uri)
+      elif type(uri) is unicode:
+        import Redland_python
+        uri = Uri(string=Redland_python.unicode_to_bytes(uri))
       else:
         raise TypeError("uri must be string or RDF.Uri")
     Redland.librdf_serializer_set_feature(self._serializer,uri._reduri,value)
@@ -1812,11 +2174,50 @@ class Serializer(object):
 # end class Serializer
 
 
+class NTriplesSerializer(Serializer):
+  """Redland N-Triples Serializer class
+
+     import RDF
+     ser=RDF.NTriplesSerializer()
+  """
+  def __init__(self):
+    return Serializer.__init__(self, name = "ntriples", mime_type = "",
+            uri = None)
+
+
+class RDFXMLSerializer(Serializer):
+  """Redland RDF/XML Serializer class
+
+     import RDF
+     ser=RDF.RDFXMLSerializer()
+  """
+  def __init__(self):
+    return Serializer.__init__(self, name = "rdfxml", mime_type = "", uri = None)
+
+class RDFXMLAbbrevSerializer(Serializer):
+  """Redland RDF/XML with abbreviations Serializer class
+
+     import RDF
+     ser=RDF.RDFXMLAbbrevSerializer()
+  """
+  def __init__(self):
+    return Serializer.__init__(self, name = "rdfxml-abbrev", mime_type = "", uri = None)
+
+class RSS10Serializer(Serializer):
+  """Redland RSS 1.0 Serializer class
+
+     import RDF
+     ser=RDF.RSS10Serializer()
+  """
+  def __init__(self):
+    return Serializer.__init__(self, name = "rss-1.0", uri = None)
+
+
 class NS(object):
   """ Redland Namespace Utility Class
 
   import RDF
-  nspace = RDF.NS("http://example.com/foo#")
+  nspace = RDF.NS("http://example.com/foo")
 
   # creates an RDF Node for http://example.com/foo#blah   
   node1 = nspace.blah
